/*******************************************************************************
 * Copyright (c) 2016 ALL4TEC & CEA LIST.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     ALL4TEC & CEA LIST - initial API and implementation
 ******************************************************************************/
package org.polarsys.esf.fmea.execution.setup;

import java.util.Iterator;
import java.util.List;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.polarsys.esf.core.profile.esfsafetyconcepts.util.ESFSafetyConceptsUtil;
import org.polarsys.esf.core.utils.ModelUtil;
import org.polarsys.esf.esfarchitectureconcepts.ISBlock;
import org.polarsys.esf.esffmea.ISBlockFMEA;
import org.polarsys.esf.esffmea.impl.ESFFMEAPackage;
import org.polarsys.esf.esffmea.impl.SBlockFMEA;
import org.polarsys.esf.esffmea.impl.SFMEA;
import org.polarsys.esf.fmea.profile.tools.util.ESFFMEAUtil;

/**
 * Setup class for starting a new fmea.
 *
 * @author $Author: ogurcan $
 * @version $Revision: 168 $
 */
public final class NewFMEASetup {

    /**
     * Default constructor, private as it's a utility class.
     */
    private NewFMEASetup() {
    }

    /**
     * Setup of a new FMEA.
     *
     * @param pSBlock The SBlock to be analysed
     */
    public static void setup(final ISBlock pSBlock) {

        Model vESFModel = ModelUtil.getWorkingModel();
        if (vESFModel != null) {

            // Verify and apply (if necessary) ESFFMEA profile
            if (vESFModel.getAppliedProfile(ESFFMEAPackage.eNS_PREFIX) == null) {
                ESFFMEAUtil.applyESFFMEAProfile(vESFModel);
            }

            // Retrieve 'SafetyArtifacts' package
            Package vSafetyArtifacts = ESFSafetyConceptsUtil.retrieveSafetyArtifactsPackage(vESFModel);

            if (vSafetyArtifacts != null) {
                // Retrieve 'FMEA' package
                Package vFMEAPackage = getFMEAPackage(vSafetyArtifacts);

                if (vFMEAPackage != null) {
                   if (getSBlockFMEA(vFMEAPackage, pSBlock) == null) {
                        // Create SBlock_FMEA in 'FMEA' package
                        ESFFMEAUtil.createSBlockFMEA(vFMEAPackage, pSBlock);
                    }
                }
            }
        }
    }

    /**
     * Get the FMEA of a SBlock (SBlockFMEA).
     *
     * @param pFMEAPackage The FMEA package
     * @param pSBlock The SBlock to be analysed
     * @return The SBlockFMEA
     */
    private static ISBlockFMEA getSBlockFMEA(final Package pFMEAPackage, final ISBlock pSBlock) {
        ISBlockFMEA vSBlockFMEA = null;
        List<Element> vElementsList = pFMEAPackage.getOwnedElements();
        Iterator<Element> vIterator = vElementsList.iterator();

        while (vSBlockFMEA == null && vIterator.hasNext()) {
            Element vElement = vIterator.next();

            if (vElement instanceof Class) {
                if (UMLUtil.getStereotypeApplication(vElement, SBlockFMEA.class) != null) {
                    vSBlockFMEA =
                        (ISBlockFMEA) UMLUtil.getStereotypeApplication(vElement, SBlockFMEA.class);
                    if (!vSBlockFMEA.getSBlock().equals(pSBlock)) {
                        vSBlockFMEA = null;
                    }
                }
            }
        }
        return vSBlockFMEA;
    }

    /**
     * Get FMEA package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     * @return The FMEA package
     */
    private static Package getFMEAPackage(final Package pSafetyArtifacts) {
        Package vFMEAPackage = null;
        Boolean vFound = false;
        List<Package> vPackagesList = pSafetyArtifacts.getNestedPackages();
        Iterator<Package> vIterator = vPackagesList.iterator();

        while (vIterator.hasNext() && !vFound) {
            Package vPackage = vIterator.next();
            if (UMLUtil.getStereotypeApplication(vPackage, SFMEA.class) != null) {
                vFound = true;
                vFMEAPackage = vPackage;
            }
        }

        if (vFMEAPackage == null) {
            ESFFMEAUtil.createFMEAPackage(pSafetyArtifacts);
            vFMEAPackage = getFMEAPackage(pSafetyArtifacts);
        }

        return vFMEAPackage;
    }
}

