/*******************************************************************************
 * Copyright (c) 2016 ALL4TEC & CEA LIST.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     ALL4TEC & CEA LIST - initial API and implementation
 ******************************************************************************/
package org.polarsys.esf.localanalysis.execution.setup;

import org.eclipse.papyrus.infra.viewpoints.policy.ViewPrototype;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.polarsys.esf.core.profile.esfsafetyconcepts.util.ESFSafetyConceptsUtil;
import org.polarsys.esf.core.utils.ModelUtil;
import org.polarsys.esf.esfarchitectureconcepts.ISBlock;
import org.polarsys.esf.esflocalanalysis.ISBlockLAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.ESFLocalAnalysisPackage;
import org.polarsys.esf.localanalysis.diagram.set.ESFLocalAnalysisDiagramSet;
import org.polarsys.esf.localanalysis.diagram.util.ESFLocalAnalysisDiagramUtil;
import org.polarsys.esf.localanalysis.profile.tools.util.ESFLocalAnalysisUtil;
import org.polarsys.esf.localanalysis.table.set.ESFLocalAnalysisTablesSet;
import org.polarsys.esf.localanalysis.table.util.ESFLocalAnalysisTablesUtil;

/**
 * Setup class for starting a new local analysis.
 *
 * @author $Author: ymunoz $
 * @version $Revision: 168 $
 */
public final class NewLocalAnalysisSetup {

    /**
     * Default constructorr, private as it's a utility class.
     */
    private NewLocalAnalysisSetup() {
    }

    /**
     * Setup of a new Local Analysis.
     *
     * @param pSBlock The SBlock to be analysed
     */
    public static void setup(final ISBlock pSBlock) {

        Model vESFModel = ModelUtil.getWorkingModel();
        if (vESFModel != null) {

            // Verify and apply (if necessary) ESFLocalAnalysis profile
            if (vESFModel.getAppliedProfile(ESFLocalAnalysisPackage.eNS_PREFIX) == null) {
                ESFLocalAnalysisUtil.applyESFLocalAnalysisProfile(vESFModel);
            }

            // Retrieve 'SafetyArtifacts' package
            Package vSafetyArtifacts = ESFSafetyConceptsUtil.retrieveSafetyArtifactsPackage(vESFModel);

            if (vSafetyArtifacts != null) {
                // Retrieve 'LocalAnalysis' package
                Package vLocalAnalysisPackage = getLocalAnalysisPackage(vSafetyArtifacts);

                if (vLocalAnalysisPackage != null) {
                    if (ESFLocalAnalysisUtil.getSBlockLAnalysisBySBlock(vLocalAnalysisPackage, pSBlock) == null) {
                        // Create SBlockLAnalysis in 'LocalAnalysis' package
                        ESFLocalAnalysisUtil.createSBlockLAnalysis(vLocalAnalysisPackage, pSBlock);
                        ISBlockLAnalysis vSBlockLAnalysis =
                            ESFLocalAnalysisUtil.getSBlockLAnalysisBySBlock(vLocalAnalysisPackage, pSBlock);
                        // Create ESFLocalAnalysis diagram
                        ESFLocalAnalysisDiagramUtil.createDiagram(
                            ESFLocalAnalysisDiagramSet.ESFLOCALANALYSIS_DIAGRAM_NAME,
                            vSBlockLAnalysis.getBase_Class());
                    }

                    // Retrieve 'SystemEventsLibrary' package
                    Package vSystemEventsLibrary = getSystemEventsLibraryPackage(vLocalAnalysisPackage);
                    ViewPrototype vTable = null;
                    if (vSystemEventsLibrary != null) {
                        vTable = ESFLocalAnalysisTablesUtil
                            .getViewPrototype(ESFLocalAnalysisTablesSet.SSYSTEMEVENTSLIBRARY_TABLE_ID);
                    }

                    // Verify if there is not a SSystemEventsLibrary table
                    if ((vTable != null) && (vTable.getViewCountOn(vSystemEventsLibrary) == 0)) {
                        // Create a SSystemEventsLibrary table
                        vTable.instantiateOn(
                            vSystemEventsLibrary,
                            ESFLocalAnalysisTablesSet.SSYSTEMEVENTSLIBRARY_TABLE_NAME
                                + vTable.getViewCountOn(vSystemEventsLibrary));
                    }

                }
            }
        }
    }

    /**
     * Get the SystemEventsLibrary package.
     *
     * @param pLocalAnalysisPackage The LocalAnalysis package
     * @return The SSystemEventsLibrary package
     */
    private static Package getSystemEventsLibraryPackage(final Package pLocalAnalysisPackage) {
        Package vSSystemEventsLibrary = ESFLocalAnalysisUtil.getSystemEventsLibraryPackage(pLocalAnalysisPackage);

        if (vSSystemEventsLibrary == null) {
            ESFLocalAnalysisUtil.createSSystemEventsLibrary(pLocalAnalysisPackage);
            vSSystemEventsLibrary = ESFLocalAnalysisUtil.getSystemEventsLibraryPackage(pLocalAnalysisPackage);
        }

        return vSSystemEventsLibrary;
    }

    /**
     * Get LocalAnalysis package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     * @return The LocalAnalysis package
     */
    private static Package getLocalAnalysisPackage(final Package pSafetyArtifacts) {
        Package vLocalAnalysisPackage = ESFLocalAnalysisUtil.getLocalAnalysisPackage(pSafetyArtifacts);

        if (vLocalAnalysisPackage == null) {
            ESFLocalAnalysisUtil.createLocalAnalysisPackage(pSafetyArtifacts);
            vLocalAnalysisPackage = ESFLocalAnalysisUtil.getLocalAnalysisPackage(pSafetyArtifacts);
        }

        return vLocalAnalysisPackage;
    }
}
